@@ -1,26 +1,39 @@
 
 c=======================================================================                      
       subroutine sprasch(datastr,imiss,ngrid,nmissing,nsubject,p,y,
-     &                   a0b0,b0,prec,psiinv,smu,tau1,tau2,
+     &                   roffset, 
+     &                   a0b0,b0,prec,psiinv,sb,smu,tau1,tau2,
      &                   mcmc,nsave,
-     &                   acrate,cpo,faccum,randsave,thetasave,
-     &                   alpha,b,bclus,beta,mu,ncluster,sigma,
-     &                   sigmainv,ss,     
-     &                   betac,ccluster,fsavet,propvf,propvr,
-     &                   prob,seed1,seed2,workmh1,workv1,grid)
+     &                   acrate,cpo,cdfsave,randsave,thetasave,
+     &                   alpha,b,bclus,beta,mu,ncluster,sigma2,
+     &                   sigmainv,ss,
+     &                   betac,ccluster,cstrt,fsavet,iflagp,prob,
+     &                   seed,
+     &                   workmhp,workvp,
+     &                   xtx,xty,grid,workcpo)
 c=======================================================================                      
 c
 c     Subroutine `sprasch' to run a Markov chain in the  
-c     semiparametric Rasch model using a Dirichlet process prior
-c     for the random effect distribution. In this routine, inference 
+c     semiparametric Rasch model. In this routine, inference 
 c     is based on  the Polya urn representation of Dirichlet process.
 c     The algorithm 8 with m=1 of Neal (2000) is used to sample the 
-c     configurations. 
+c     configurations.
 c
-c     Copyright: Alejandro Jara, 2006-2009
+c     Copyright: Alejandro Jara, 2006 - 2009
 c
-c     Version 1.0: 
-c     Last modification: 09-04-2007.
+c     Version 3.0: 
+c
+c     Last modification: 05-09-2009.
+c
+c     Changes and Bug fixes: 
+c
+c     Version 2.0 to Version 3.0:
+c          - Add computation of functionals.
+c          - Improve efficiency of DP part.
+c          - CDF samples are returned now.
+c
+c     Version 1.0 to Version 2.0:
+c          - Add offset.
 c
 c     This program is free software; you can redistribute it and/or modify
 c     it under the terms of the GNU General Public License as published by
@@ -40,11 +53,11 @@ c     The author's contact information:
 c
 c      Alejandro Jara
 c      Department of Statistics
-c      Facultad de Ciencias Fsicas y Matemticas
-c      Universidad de Concepcin
+c      Facultad de Ciencias Fisicas y Matematicas
+c      Universidad de Concepcion
 c      Avenida Esteban Iturra S/N
 c      Barrio Universitario
-c      Concepcin
+c      Concepcion
 c      Chile
 c      Voice: +56-41-2203163  URL  : http://www2.udec.cl/~ajarav
 c      Fax  : +56-41-2251529  Email: ajarav@udec.cl
@@ -61,6 +74,8 @@ c        nmissing    :  integer giving the number of missing
 c                       observations.
 c        nsubject    :  integer giving the number of subjects.
 c        p           :  integer giving the number of items.
+c        roffset     :  real matrix giving the offsets,
+c                       roffset(nsubject,p).
 c        y           :  integer matrix giving the response variable,
 c                       y(nsubject,p).
 c
@@ -105,15 +120,15 @@ c
 c        acrate      :  real vector giving the MH acceptance rate, 
 c                       acrate(2). 
 c        cpo         :  real matrix giving the cpo, cpo(nsubject,p).
-c        faccum      :  real vector giving the cdf estimate at the
-c                       grid, faccum(ngrid).
+c        cdfsave     :  real matrix containing the mcmc samples for the 
+c                       cdf, cdfsave(nsave,ngrid).
 c        randsave    :  real matrix containing the mcmc samples for
 c                       the random effects and prediction,
 c                       randsave(nsave,nsubject+1)
 c        thetasave   :  real matrix containing the mcmc samples for
 c                       the fixed effects,and mean and covariance of
 c                       the baseline distribution, 
-c                       thetsave(nsave,p+3).
+c                       thetsave(nsave,p+5).
 c
 c-----------------------------------------------------------------------
 c
@@ -153,27 +168,26 @@ c                       candidate for difficulty parameters,
 c                       betac(p-1).
 c        ccluster    :  integer vector indicating the number of
 c                       subjects in each cluster, ccluster(nsubject).
+c        detlog      :  real used to save the log-determinant in a
+c                       matrix inversion process.
 c        dispcount   :  index. 
 c        evali       :  integer indicator used in updating the state.
-c        fsavet      :  integer vector used to store the number of
-c                       observations to calculate the predictive
-c                       cdf distribution, fsavet(ngrid).
+c        fsavet      :  real vector used to store the cdf, 
+c                       fsavet(ngrid).
 c        grid        :  real vector giving the grid where the density
 c                       estimate is evaluated, grid(ngrid) .
 c        i           :  index. 
 c        ii          :  index. 
+c        iflag       :  integer vector used to invert the of the lhs
+c                       least square solution for the difficulties,
+c                       iflag(p-1).
 c        isave       :  index. 
 c        iscan       :  index. 
 c        j           :  index. 
 c        k           :  index. 
 c        l           :  index.
 c        prob        :  real vector used to update the cluster 
-c                       structure, prob(nsubject+2).
-c        propvf      :  real matrix giving the variance of the normal
-c                       proposal for the difficulty parameters,
-c                       propvf(p-1,p-1).
-c        propvr      :  real giving the variance of the normal
-c                       proposal for the random effects, propvr. 
+c                       structure, prob(nsubject+100).
 c        ns          :  integer indicator used in updating the state. 
 c        nscan       :  integer indicating the total number of MCMC
 c                       scans.
@@ -188,22 +202,31 @@ c        thetac      :  real used to save randomnly generated
 c                       random effects, thetac.
 c        tmp1        :  real used to accumulate quantities. 
 c        tmp2        :  real used to accumulate quantities.
-c        workmh1     :  real vector used to update the difficulties,
-c                       workmh1((p-1)*p/2).
-c        workv1      :  real vector used to update the difficulties,
-c                       workv1(p-1).
-c
+c        workmhp     :  real vector used to update the difficulties,
+c                       workmhp((p-1)*p/2).
+c        workvp      :  real vector used to update the difficulties,
+c                       workvp(p-1).
+c        xtx         :  real matrix givind the product X^tX, 
+c                       xtx(p-1,p-1).
+c        xty         :  real vector used to save the product 
+c                       Xt(Y-Zb), xty(p-1).
+c        zty         :  real used to save the product 
+c                       Zt(Y-Xbeta), zty.
+c        ztz         :  real used to save the product 
+c                       ZtSigma^1Z, ztz.
+c        ztzinv      :  real used to save the inverted 
+c                       ztz, ztzinv.
 c=======================================================================                  
-
       implicit none 
 
 c+++++Data
       integer imiss,ngrid,nmissing,nsubject,p
       integer datastr(nmissing,2),y(nsubject,p)
+      real*8 roffset(nsubject,p)
 
 c+++++Prior 
       real*8 aa0,ab0,a0b0(2),b0(p-1),prec(p-1,p-1),psiinv
-      real*8 smu
+      real*8 sb(p-1),smu
       real*8 tau1,tau2
 
 c+++++MCMC parameters
@@ -211,56 +234,72 @@ c+++++MCMC parameters
 
 c+++++Output
       real*8 acrate(2)
-      real*8 cpo(nsubject,p),faccum(ngrid)
+      real*8 cpo(nsubject,p)
+      real*8 cdfsave(nsave,ngrid)
       real*8 randsave(nsave,nsubject+1)
-      real*8 thetasave(nsave,p+3)
+      real*8 thetasave(nsave,p+5)
 
 c+++++Current values of the parameters
       integer ncluster,ss(nsubject)
       real*8 alpha,beta(p-1),b(nsubject)
-      real*8 bclus(nsubject)
-      real*8 mu,sigma,sigmainv
+      real*8 bclus(nsubject+100)
+      real*8 mu,sigma2,sigmainv
 
-c+++++Working space - General
-      integer i,ii,j,k
-      integer sprint
-      real*8 eta,cdfnorm,dbin
-      real*8 mean
-      real*8 tmp1,tmp2
+c+++++Working space - Loops
+      integer ii,i,j,k
 
 c+++++Working space - Random effects
-      integer fsavet(ngrid)
-      real*8 grid(ngrid),thetac,propvr
-
-c+++++Working space - Configurations
-      integer ccluster(nsubject),evali 
-      integer ns
-      integer since
-      real*8 prob(nsubject+2)
-      real*8 theta
+      real*8 fsavet(ngrid)
+      real*8 cdfnorm,dnrm,grid(ngrid),thetac
+      real*8 zty,ztz,ztzinv
 
 c+++++Working space - RNG
-      integer seed1,seed2
+      integer rpois,seed(2),seed1,seed2
       real*8 rgamma,rnorm
       real runif
 
 c+++++Working space - MCMC
       integer iscan,isave,nscan
-      integer skipcount,dispcount
+      integer sprint,skipcount,dispcount
+      real*8 dbin
+      real*8 logcgkn,logcgko
+      real*8 loglikn,logliko
+      real*8 logpriorn,logprioro
+      real*8 ratio
+      
+c+++++Working space - Configurations
+      integer ccluster(nsubject),evali,isample,ok 
+      integer cstrt(nsubject,nsubject)
+      integer ns
+      integer since
+      real*8 prob(nsubject+2)
+      real*8 tmp1,tmp2,tmp3
+      real*8 theta
 
 c+++++Working space - Difficulty parameters
+      integer iflagp(p-1)
       real*8 betac(p-1)
-      real*8 propvf(p-1,p-1)
-      real*8 workmh1((p-1)*p/2),workv1(p-1)
-      
+      real*8 detlog
+      real*8 xtx(p-1,p-1),xty(p-1)
+      real*8 workmhp((p-1)*p/2)
+      real*8 workvp(p-1)
+
+c+++++Working space - CPO
+      real*8 workcpo(nsubject,p)
+
 c+++++Working space - GLM part
       integer yij
       real*8 acrate2
-      real*8 logp
+      real*8 eta,gprime,mean,offset,ytilde
 
 c+++++CPU time
       real*8 sec00,sec0,sec1,sec
 
+c++++ DP (functional parameter)
+      real*8 eps,rbeta,weight
+      real*8 mureal,sigma2real,mu2real
+      parameter(eps=0.01)
+
 c++++ parameters
       nburn=mcmc(1)
       nskip=mcmc(2)
@@ -269,14 +308,17 @@ c++++ parameters
       ab0=a0b0(2)
       
 c++++ set random number generator
+      seed1=seed(1)
+      seed2=seed(2)
       
       call setall(seed1,seed2)
-
+     
 c++++ cluster structure
       do i=1,nsubject
          ccluster(ss(i))=ccluster(ss(i))+1
+         cstrt(ss(i),ccluster(ss(i)))=i
       end do
-     
+      sigmainv=1.d0/sigma2 
       
 c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 c++++ start the MCMC algorithm
@@ -286,7 +328,7 @@ c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       skipcount=0
       dispcount=0
       nscan=nburn+(nskip+1)*(nsave)
-      
+
       call cpu_time(sec0)
       sec00=0.d0
       
@@ -303,13 +345,12 @@ c++++++++++++++++++++++++++++++++++
                i=datastr(ii,1)
                j=datastr(ii,2)
                if(j.eq.1)then
-                 eta=b(i)
+                 eta=b(i)+roffset(i,j)
                 else
-                 eta=b(i)-beta(j-1)
+                 eta=b(i)-beta(j-1)+roffset(i,j)
                end if
-               
+
                mean=exp(eta)/(1.d0+exp(eta))
-               
                call rbinom(1,mean,evali)
                y(i,j)=evali
             end do
@@ -323,54 +364,128 @@ c++++++++++++++++++++++++++++++++++
 c+++++++ difficulty parameters
 c++++++++++++++++++++++++++++++++++
 
-         logp=0.d0 
-
-         call rmvnorm(p-1,beta,propvf,workmh1,workv1,betac)
-
-         do i=1,nsubject
+         do i=1,p-1
             do j=1,p-1
-               eta=b(i)-betac(j) 
-               mean=exp(eta)/(1.d0+exp(eta))
-               logp=logp+dbin(dble(y(i,j+1)),1.d0,mean,1)
-            end do
-         
-            do j=1,p-1
-               eta=b(i)-beta(j) 
-               mean=exp(eta)/(1.d0+exp(eta))
-               logp=logp-dbin(dble(y(i,j+1)),1.d0,mean,1)
+               xtx(i,j)=prec(i,j)
             end do
+            xty(i)=sb(i)
          end do
 
-         tmp1=0.d0
-         tmp2=0.d0
+         logliko=0.d0         
+         
+         do i=1,nsubject
+            do j=1,p-1
+               yij=y(i,j+1)
+               eta=b(i)-beta(j)+roffset(i,j+1) 
+               offset=b(i)+roffset(i,j+1) 
+
+               mean=exp(eta)/(1.d0+exp(eta))
+               logliko=logliko+dbin(dble(yij),1.d0,mean,1)
+
+               tmp1=mean*(1.0d0-mean)
+               gprime=1.d0/tmp1
+
+               ytilde=eta+(dble(yij)-mean)*gprime-offset
+               xtx(j,j)=xtx(j,j)+1.d0/gprime
+               xty(j)=xty(j)-ytilde/gprime
+
+            end do
+         end do
+         
+         call inverse(xtx,p-1,iflagp)      
+            
+         do i=1,p-1
+            tmp1=0.d0
+            do j=1,p-1
+               tmp1=tmp1+xtx(i,j)*xty(j) 
+            end do
+            workvp(i)=tmp1
+         end do
+
+         call rmvnorm(p-1,workvp,xtx,workmhp,xty,betac)
+
+c+++++++ evaluating the candidate generating kernel
+
+         call dmvnd(p-1,betac,workvp,xtx,logcgko,iflagp)
+
+c+++++++ prior ratio
+
+         logprioro=0.d0
+         logpriorn=0.d0
          
          do i=1,p-1
             do j=1,p-1
-               tmp1=tmp1+(betac(i)-b0(i))* 
-     &              prec(i,j)      *
-     &              (betac(j)-b0(j))
-
-               tmp2=tmp2+(beta(i)-b0(i))* 
-     &              prec(i,j)      *
-     &              (beta(j)-b0(j))
+               logpriorn=logpriorn+(betac(i)-b0(i))* 
+     &                    prec(i,j)      *
+     &                   (betac(j)-b0(j))
 
+               logprioro=logprioro+(beta(i) -b0(i))* 
+     &                    prec(i,j)      *
+     &                   (beta(j) -b0(j))
             end do
          end do
-      
-         logp=logp-0.5d0*tmp1+0.5d0*tmp2 
          
-c+++++++ mh step
+         logpriorn=-0.5d0*logpriorn
+         logprioro=-0.5d0*logprioro
 
-         if(log(dble(runif())).lt.logp)then
+c+++++++ candidate generating kernel contribution
+
+         do i=1,p-1
+            do j=1,p-1
+               xtx(i,j)=prec(i,j)
+            end do
+            xty(i)=sb(i)
+         end do
+
+         loglikn=0.d0         
+
+         do i=1,nsubject
+            do j=1,p-1
+               yij=y(i,j+1)
+               eta=b(i)-betac(j)+roffset(i,j+1)  
+               offset=b(i)+roffset(i,j+1) 
+               mean=exp(eta)/(1.d0+exp(eta))
+               loglikn=loglikn+dbin(dble(yij),1.d0,mean,1)
+
+               tmp1=mean*(1.0d0-mean)
+               gprime=1.d0/tmp1
+
+               ytilde=eta+(dble(yij)-mean)*gprime-offset
+               xtx(j,j)=xtx(j,j)+1.d0/gprime
+               xty(j)=xty(j)-ytilde/gprime
+            end do
+         end do
+         
+         call inverse(xtx,p-1,iflagp)      
+            
+         do i=1,p-1
+            tmp1=0.d0
+            do j=1,p-1
+               tmp1=tmp1+xtx(i,j)*xty(j) 
+            end do
+            workvp(i)=tmp1
+         end do
+
+c+++++++ evaluating the candidate generating kernel
+
+         call dmvnd(p-1,beta,workvp,xtx,logcgkn,iflagp)
+
+c+++++++ mh step
+           
+         ratio=loglikn-logliko+logcgkn-logcgko+
+     &         logpriorn-logprioro
+
+         if(log(runif()).lt.ratio)then
             acrate(1)=acrate(1)+1.d0
             do i=1,p-1
                beta(i)=betac(i) 
             end do
          end if
-         
+            
 c+++++++ check if the user has requested an interrupt
          call rchkusr()
 
+
 c++++++++++++++++++++++++++++++++++         
 c+++++++ random effects 
 c++++++++++++++++++++++++++++++++++
@@ -383,202 +498,221 @@ c++++++++++++++++++++++++++++++
          
             ns=ccluster(ss(i))
 
-c++++++++++ observation in cluster with more than 1 element
-             
             if(ns.gt.1)then
- 
+               isample=1
                ccluster(ss(i))=ccluster(ss(i))-1 
-               
-               do j=1,ncluster
-                  
-                  tmp1=0.d0
-                  
-                  yij=y(i,1)
-                  eta=bclus(j)
 
-                  mean=exp(eta)/(1.d0+exp(eta))
-                  tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
-
-                  do k=1,p-1
-                     yij=y(i,k+1)
-                     eta=bclus(j)-beta(k)
-                     mean=exp(eta)/(1.d0+exp(eta))                     
-
-                     tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
-                  end do
-
-                  prob(j)=exp(log(dble(ccluster(j)))+
-     &                        tmp1)
-               end do
-               
-               theta=rnorm(mu,sigma)
-
-               tmp1=0.d0
-               
-               yij=y(i,1)
-               eta=theta
-               mean=exp(eta)/(1.d0+exp(eta))
-               
-               tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
-
-               do k=1,p-1
-                  yij=y(i,k+1)
-                  eta=theta-beta(k)
-                  mean=exp(eta)/(1.d0+exp(eta))
-               
-                  tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
+               j=1
+               ok=0
+               do while(ok.eq.0.and.j.le.ns)
+                  if(cstrt(ss(i),j).eq.i)ok=j
+                  j=j+1
                end do
 
-               prob(ncluster+1)=exp(log(alpha)+tmp1)
-               
-               call simdisc(prob,nsubject+2,ncluster+1,evali)
+               do j=ok,ns-1
+                  cstrt(ss(i),j)=cstrt(ss(i),j+1)
+               end do
 
-               if(evali.le.ncluster)then
-                  ss(i)=evali
-                  ccluster(evali)=ccluster(evali)+1
-               end if   
-               if(evali.gt.ncluster)then
-                  ncluster=ncluster+1
-                  ss(i)=ncluster
-                  ccluster(ncluster)=1
-                  bclus(ncluster)=theta
-               end if               
-            end if
-
-c++++++++++ subject in cluster with only 1 observation
-             
-            if(ns.eq.1)then
-                
+             else
+               isample=2
                since=ss(i)
-                
+    
                if(since.lt.ncluster)then
-                   call relabelrasch(i,since,nsubject,ncluster,
-     &                          ccluster,ss,bclus)                   
-               end if
 
+                  call relabelrasch(i,since,nsubject,ncluster,
+     &                              ccluster,ss,cstrt,bclus)
+
+               end if
                ccluster(ncluster)=ccluster(ncluster)-1 
                ncluster=ncluster-1
+            end if 
 
-
-               do j=1,ncluster
+            do j=1,ncluster
                   
-                  tmp1=0.d0
-                  
-                  yij=y(i,1)
-                  eta=bclus(j)
-
-                  mean=exp(eta)/(1.d0+exp(eta))                     
-
-                  tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
-                  
-                  do k=1,p-1
-                     yij=y(i,k+1)
-                     eta=bclus(j)-beta(k)
-
-                     mean=exp(eta)/(1.d0+exp(eta))                     
-
-                     tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
-
-                  end do
-
-                  prob(j)=exp(log(dble(ccluster(j)))+
-     &                        tmp1)
-               end do
-               
-               theta=b(i)
-
                tmp1=0.d0
-               
+                  
                yij=y(i,1)
-               eta=theta
-               mean=exp(eta)/(1.d0+exp(eta))                     
-
+               eta=bclus(j)+roffset(i,1) 
+               mean=exp(eta)/(1.d0+exp(eta))
                tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
-               
+
                do k=1,p-1
                   yij=y(i,k+1)
-                  eta=theta-beta(k)
-
-                  mean=exp(eta)/(1.d0+exp(eta))                     
-
+                  eta=bclus(j)-beta(k)+roffset(i,k+1) 
+                  mean=exp(eta)/(1.d0+exp(eta))
                   tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
                end do
 
-               prob(ncluster+1)=exp(log(alpha)+tmp1)
+               prob(j)=exp(log(dble(ccluster(j)))+
+     &                        tmp1)
+            end do
                
-               call simdisc(prob,nsubject+2,ncluster+1,evali)
+            if(isample.eq.1)then
+               theta=rnorm(mu,sqrt(sigma2))
+               bclus(ncluster+1)=theta
+            end if 
+            
+            theta=bclus(ncluster+1)
+            tmp1=0.d0
+               
+            yij=y(i,1)
+            eta=theta+roffset(i,1) 
+            mean=exp(eta)/(1.d0+exp(eta))
+            tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
 
-               if(evali.le.ncluster)then
-                  ss(i)=evali
-                  ccluster(evali)=ccluster(evali)+1
-               end if   
-               if(evali.gt.ncluster)then
-                  ncluster=ncluster+1
-                  ss(i)=ncluster
-                  ccluster(ncluster)=1
-                  bclus(ncluster)=theta
-               end if      
+            do k=1,p-1
+               yij=y(i,k+1)
+               eta=theta-beta(k)+roffset(i,k+1) 
+               mean=exp(eta)/(1.d0+exp(eta))
+               tmp1=tmp1+dbin(dble(yij),1.d0,mean,1)
+            end do
 
+            prob(ncluster+1)=alpha*exp(tmp1)
+               
+            call simdisc(prob,nsubject+100,ncluster+1,evali)
+
+            ss(i)=evali
+            ccluster(evali)=ccluster(evali)+1
+            cstrt(evali,ccluster(evali))=i
+
+            if(evali.gt.ncluster)then
+               ncluster=ncluster+1
             end if
 
          end do
 
+
 c++++++++++++++++++++++++++++++
 c+++++++ b) Resampling step
 c++++++++++++++++++++++++++++++
 
          acrate2=0.d0
          
-         do ii=1,ncluster
-         
-            thetac=rnorm(bclus(ii),sqrt(propvr))
-            logp=0.d0
+         do i=1,ncluster
 
-            do i=1,nsubject
-               if(ii.eq.ss(i))then
-                 do j=1,p
-                    if(j.eq.1)then
-                       eta=bclus(ii)
-                      else
-                       eta=bclus(ii)-beta(j-1)
-                    end if
-                    yij=y(i,j)
-                    mean=exp(eta)/(1.d0+exp(eta))
-                    logp=logp-dbin(dble(yij),1.d0,mean,1)
+c++++++++++ check if the user has requested an interrupt
+            call rchkusr()
 
-                    if(j.eq.1)then
-                       eta=thetac
-                      else
-                       eta=thetac-beta(j-1)
-                    end if
-                    yij=y(i,j)
-                    mean=exp(eta)/(1.d0+exp(eta))
-                    logp=logp+dbin(dble(yij),1.d0,mean,1)
-                 end do
-               end if
+            ztz=1.d0/sigma2
+            zty=mu/sigma2
+
+            logliko=0.d0                     
+
+            ns=ccluster(i)
+            do j=1,ns
+
+c+++++++++++++ check if the user has requested an interrupt
+               call rchkusr()
+
+               ii=cstrt(i,j)
+
+               do k=1,p
+                  if(k.eq.1)then
+                     eta=bclus(i)+roffset(ii,k)
+                     offset=roffset(ii,k)
+                    else
+                     eta=bclus(i)-beta(k-1)+roffset(ii,k)
+                     offset=roffset(ii,k)-beta(k-1)
+                  end if
+                    
+                  yij=y(ii,k)
+
+                  mean=exp(eta)/(1.d0+exp(eta))
+                  logliko=logliko+dbin(dble(yij),1.d0,mean,1)
+
+                  tmp1=mean*(1.0d0-mean)
+                  gprime=1.d0/tmp1
+
+                  ytilde=eta+(dble(yij)-mean)*gprime-offset
+                  ztz=ztz+1.d0/gprime
+                  zty=zty+ytilde/gprime
+
+               end do
             end do
-            
+
+            ztz=1.d0/ztz
+            zty=ztz*zty
+
+            thetac=rnorm(zty,sqrt(ztz))
+
+c++++++++++ evaluating the candidate generating kernel
+
+            logcgko=dnrm(thetac,zty,sqrt(ztz),1)
+
 c++++++++++ prior ratio
 
-            tmp1=(thetac-mu)* 
+            logprioro=0.d0
+            logpriorn=0.d0
+         
+            logpriorn=(thetac-mu)* 
      &            sigmainv*
      &           (thetac-mu)
 
-            tmp2=(bclus(ii)-mu)* 
+            logprioro=(bclus(i)-mu)* 
      &            sigmainv*
-     &           (bclus(ii)-mu)
+     &           (bclus(i)-mu)
             
-            logp=logp-0.5d0*tmp1+0.5d0*tmp2 
+            logpriorn=-0.5d0*logpriorn
+            logprioro=-0.5d0*logprioro
+
+
+c++++++++++ candidate generating kernel contribution
+
+            ztz=1.d0/sigma2
+            zty=mu/sigma2
+
+            loglikn=0.d0                     
             
+            ns=ccluster(i)
+            do j=1,ns
+
+c+++++++++++++ check if the user has requested an interrupt
+               call rchkusr()
+
+               ii=cstrt(i,j)
+
+               do k=1,p
+                  if(k.eq.1)then
+                     eta=thetac+roffset(ii,k)
+                     offset=roffset(ii,k)
+                    else
+                     eta=thetac-beta(k-1)+roffset(ii,k)
+                     offset=roffset(ii,k)-beta(k-1)
+                  end if
+                    
+                  yij=y(ii,k)
+
+                  mean=exp(eta)/(1.d0+exp(eta))
+                  loglikn=loglikn+dbin(dble(yij),1.d0,mean,1)
+
+                  tmp1=mean*(1.0d0-mean)
+                  gprime=1.d0/tmp1
+
+                  ytilde=eta+(dble(yij)-mean)*gprime-offset
+                  ztz=ztz+1.d0/gprime
+                  zty=zty+ytilde/gprime
+               end do
+            end do
+
+            ztz=1.d0/ztz
+            zty=ztz*zty
+
+c++++++++++ evaluating the candidate generating kernel
+
+            theta=bclus(i)
+            logcgkn=dnrm(theta,zty,sqrt(ztz),1)                 
+
 c++++++++++ mh step
+           
+            ratio=loglikn-logliko+logcgkn-logcgko+
+     &            logpriorn-logprioro
 
-            if(log(dble(runif())).lt.logp)then
+            if(log(runif()).lt.ratio)then
                acrate2=acrate2+1.d0
-               bclus(ii)=thetac
+               bclus(i)=thetac
             end if
-
          end do
-         
+
          acrate(2)=acrate(2)+acrate2/dble(ncluster)
 
          do i=1,nsubject
@@ -596,16 +730,12 @@ c+++++++ check if the user has requested an interrupt
          if(psiinv.gt.0.d0)then
             tmp2=smu
             tmp1=1.d0/(sigmainv*dble(ncluster)+psiinv)
-
             do i=1,ncluster
                tmp2=tmp2+sigmainv*bclus(i)
             end do
-
             mean=tmp1*tmp2
-
             mu=rnorm(mean,sqrt(tmp1))
          end if 
-          
 
 c+++++++ check if the user has requested an interrupt
          call rchkusr()
@@ -615,14 +745,13 @@ c+++++++ check if the user has requested an interrupt
             do i=1,ncluster
                tmp1=tmp1+(bclus(i)-mu)*(bclus(i)-mu)                   
             end do
-   
-            tmp2=1.d0/
+
+            sigma2=1.d0/
      &             rgamma(0.5d0*(dble(ncluster)+tau1),0.5d0*(tmp1+tau2))
      
-            sigma=sqrt(tmp2)
-            sigmainv=1.d0/tmp2
+            sigmainv=1.d0/sigma2
          end if   
-
+         
 c++++++++++++++++++++++++++++++++++         
 c+++++++ Precision parameter
 c++++++++++++++++++++++++++++++++++
@@ -646,79 +775,152 @@ c+++++++++++++ difficulty parameters
                   thetasave(isave,i)=beta(i)
                end do
 
+c+++++++++++++ Partially sampling the DP for mean and variance
+
+               do i=1,ngrid
+                  fsavet(i)=0.d0
+               end do
+
+               mureal=0.d0 
+               mu2real=0.d0
+               sigma2real=0.d0
+
+               do i=1,ncluster
+                   prob(i)=real(ccluster(i))/(alpha+real(nsubject))
+               end do
+               prob(ncluster+1)=alpha/(alpha+dble(nsubject))
+               call simdisc(prob,nsubject+100,ncluster+1,evali)
+
+               if(evali.le.ncluster)then
+                  theta=bclus(evali)
+               end if
+               if(evali.eq.ncluster+1)then 
+                  theta=rnorm(mu,sqrt(sigma2))
+               end if
+
+               tmp1=rbeta(1.d0,alpha+dble(nsubject))
+               tmp2=tmp1
+               weight=(1.d0-tmp1)
+
+               mureal=tmp1*theta 
+               mu2real=tmp1*theta*theta
+               do j=1,ngrid
+                  if(theta.le.grid(j))then
+                     fsavet(j)=fsavet(j)+tmp1
+                  end if
+               end do
+
+               do while((1.0-tmp2).gt.eps)
+                  call rchkusr()
+
+                  tmp3=rbeta(1.d0,alpha+real(nsubject))
+                  tmp1=weight*tmp3
+                  weight=weight*(1.0-tmp3)
+
+                  call simdisc(prob,nsubject+100,ncluster+1,evali)
+
+                  if(evali.le.ncluster)then
+                     theta=bclus(evali)
+                  end if
+                  if(evali.eq.ncluster+1)then 
+                     theta=rnorm(mu,sqrt(sigma2))
+                  end if
+
+                  mureal=mureal+tmp1*theta 
+                  mu2real=mu2real+tmp1*theta*theta
+
+                  do j=1,ngrid
+                     if(theta.le.grid(j))then
+                        fsavet(j)=fsavet(j)+tmp1
+                     end if
+                  end do
+                  tmp2=tmp2+tmp1
+               end do
+
+               tmp3=1.d0
+               tmp1=weight*tmp3
+               call simdisc(prob,nsubject+100,ncluster+1,evali)
+
+               if(evali.le.ncluster)then
+                  theta=bclus(evali)
+               end if
+               if(evali.eq.ncluster+1)then 
+                  theta=rnorm(mu,sqrt(sigma2))
+               end if
+
+               mureal=mureal+tmp1*theta 
+               mu2real=mu2real+tmp1*theta*theta
+
+               do j=1,ngrid
+                  if(theta.le.grid(j))then
+                     fsavet(j)=fsavet(j)+tmp1
+                  end if
+               end do
+
+               sigma2real=mu2real-mureal*mureal
+
+               thetasave(isave,p)=mureal
+               thetasave(isave,p+1)=sigma2real
+
+               do i=1,ngrid
+                  cdfsave(isave,i)=fsavet(i)
+               end do
+
 c+++++++++++++ baseline mean
 
-               thetasave(isave,p)=mu
+               thetasave(isave,p+2)=mu
 
 c+++++++++++++ baseline stansard deviation
 
-               thetasave(isave,p+1)=sigma
+               thetasave(isave,p+3)=sigma2
 
 c+++++++++++++ cluster information
 
-               thetasave(isave,p+2)=ncluster
-               thetasave(isave,p+3)=alpha
-               
+               thetasave(isave,p+4)=ncluster
+               thetasave(isave,p+5)=alpha
+
 c+++++++++++++ random effects
 
                do i=1,nsubject
                   randsave(isave,i)=b(i)
                end do
 
-
 c+++++++++++++ predictive information
 
-               do i=1,ngrid
-                  fsavet(i)=0
-               end do
-
                do i=1,ncluster
                   prob(i)=dble(ccluster(i))/(alpha+dble(nsubject))
-                  
-                  j=1
-                  do while(bclus(i).le.grid(j).and.j.le.ngrid)
-                     fsavet(j)=fsavet(j)+ccluster(i)
-                     j = j + 1
-                  end do
-
                end do
-
-               do j=1,ngrid
-                  faccum(j)=faccum(j)+
-     &             ( alpha*cdfnorm(grid(j),mu,sigma,1,0) +
-     &               dble(fsavet(j)))/
-     &              (alpha+dble(nsubject))  
-
-               end do
-
                prob(ncluster+1)=alpha/(alpha+dble(nsubject))
 
-               call simdisc(prob,nsubject+2,ncluster+1,evali)
+               call simdisc(prob,nsubject+100,ncluster+1,evali)
                
                if(evali.le.ncluster)then
                   theta=bclus(evali)
                end if
                if(evali.gt.ncluster)then
-                  theta=rnorm(mu,sigma)
+                  theta=rnorm(mu,sqrt(sigma2))
                end if
-               
                randsave(isave,nsubject+1)=theta
 
 c+++++++++++++ cpo
 
+               tmp2=0.d0
                do i=1,nsubject
                   do j=1,p
                      yij=y(i,j)
                      if(j.eq.1)then
-                       eta=b(i)
+                       eta=b(i)+roffset(i,j)
                       else
-                       eta=b(i)-beta(j-1)
+                       eta=b(i)-beta(j-1)+roffset(i,j)
                      end if  
+
                      mean=exp(eta)/(1.d0+exp(eta))
-                     tmp1=dbin(dble(yij),1.d0,mean,1)
-                     cpo(i,j)=cpo(i,j)+1.0d0/exp(tmp1)   
+                     tmp1=dbin(dble(yij),1.d0,mean,0)
+                     cpo(i,j)=cpo(i,j)+1.0d0/tmp1   
+                     tmp2=tmp2+log(dble(isave)/cpo(i,j))
                   end do
                end do
+c               call dblepr("LPML",-1,tmp2,1)
 
 c+++++++++++++ print
                skipcount = 0
@@ -735,51 +937,16 @@ c+++++++++++++ print
 
       end do
       
-      do i=1,2
-         acrate(i)=acrate(i)/dble(nscan)    
-      end do   
+      acrate(1)=acrate(1)/dble(nscan)    
+      acrate(2)=acrate(2)/dble(nscan)    
       
       do i=1,nsubject
          do j=1,p
             cpo(i,j)=dble(nsave)/cpo(i,j)
          end do   
       end do
-      
-      do i=1,ngrid
-         faccum(i)=faccum(i)/dble(nsave)
-      end do
-            
+             
       return
       end
 
 
-c=======================================================================      
-      subroutine relabelrasch(ind,since,nsubject,ncluster,ccluster,
-     &                        ss,bclus)
-c=======================================================================
-c     relabel the clusters after the elimination of one of them
-c     A.J.V., 2006
-      implicit none
-      integer i,j,ind,since,nsubject,ncluster,ccluster(nsubject)
-      integer ss(nsubject)
-      real*8 bclus(nsubject),theta
-
-      theta=bclus(since)
-      
-      do i=since+1,ncluster
-         do j=1,nsubject
-            if(ss(j).eq.i)then
-               ss(j)=i-1 
-            end if
-         end do
-         bclus(i-1)=bclus(i)
-         ccluster(i-1)=ccluster(i)
-      end do
-      
-      ss(ind)=ncluster
-      
-      bclus(ncluster)=theta
-      ccluster(ncluster)=1
-      
-      return
-      end  
