@@ -43,11 +43,11 @@ c     The author's contact information:
 c
 c      Alejandro Jara
 c      Department of Statistics
-c      Facultad de Ciencias Fsicas y Matemticas
-c      Universidad de Concepcin
+c      Facultad de Ciencias Fisicas y Matematicas
+c      Universidad de Concepcion
 c      Avenida Esteban Iturra S/N
 c      Barrio Universitario
-c      Concepcin
+c      Concepcion
 c      Chile
 c      Voice: +56-41-2203163  URL  : http://www2.udec.cl/~ajarav
 c      Fax  : +56-41-2251529  Email: ajarav@udec.cl
@@ -385,10 +385,6 @@ c++++++++++++++++++++++++++++++++
                logliko=logliko+dbin(dble(yij),dble(nij),mean,1)
 
                tmp1=mean*(1.0d0-mean)
-               if(tmp1.le.0.0001d0)then
-                  tmp1=0.0001d0
-               end if   
-               
                gprime=1.d0/(dble(nij)*tmp1)
 
                mean=dble(nij)*exp(eta)/(1.d0+exp(eta))
@@ -453,10 +449,6 @@ c++++++++++ evaluating the likelihood
                loglikn=loglikn+dbin(dble(yij),dble(nij),mean,1)
 
                tmp1=mean*(1.0d0-mean)
-               if(tmp1.le.0.0001d0)then
-                  tmp1=0.0001d0
-               end if   
-               
                gprime=1.d0/(dble(nij)*tmp1)
 
                mean=dble(nij)*exp(eta)/(1.d0+exp(eta))
@@ -581,10 +573,6 @@ c              end do
                  logliko=logliko+dbin(dble(yij),dble(nij),mean,1)
 
                  tmp1=mean*(1.0d0-mean)
-                 if(tmp1.le.0.0001d0)then
-                    tmp1=0.0001d0
-                 end if   
-                 
                  gprime=1.d0/(dble(nij)*tmp1)
  
                  mean=dble(nij)*exp(eta)/(1.d0+exp(eta))
@@ -675,10 +663,6 @@ c              end do
                  loglikn=loglikn+dbin(dble(yij),dble(nij),mean,1)
 
                  tmp1=mean*(1.0d0-mean)
-                 if(tmp1.le.0.0001d0)then
-                    tmp1=0.0001d0
-                 end if   
-                 
                  gprime=1.d0/(dble(nij)*tmp1)
  
                  mean=dble(nij)*exp(eta)/(1.d0+exp(eta))
